#!/bin/bash
#
# dir-add-delete.sh
#
# Test whether adding and removing a dir on trunk, and adding it on
# branch introduces subtree merge info.

set -e

SERVER_DIR=server
CLIENT_DIR=client

if [ $# != 1 ]; then
    echo "usage: $0 <path to svn>"
    exit
fi

SVN=$1
SVNADMIN="$(dirname $SVN)/svnadmin"

SERVER_URL="file:///$PWD/$SERVER_DIR"

# Create the repo and enter it.
$SVNADMIN create $SERVER_DIR
$SVN checkout $SERVER_URL $CLIENT_DIR
cd $CLIENT_DIR

# Create the trunk/branches structure
mkdir trunk
mkdir branches
$SVN add trunk branches
$SVN commit -m 'Create initial structure'

# Make a branch from the trunk.
$SVN copy ^/trunk ^/branches/branch -m 'Create branch from trunk'

# Check out the trunk.
$SVN switch --ignore-ancestry ^/trunk ||
    $SVN switch ^/trunk

# Create a file in a directory and commit it on the trunk.
mkdir dir
echo hey >dir/file.txt
$SVN add dir
$SVN commit -m 'Created dir/file.txt on trunk'

# Remove the newly-created dir on the trunk.
$SVN update
$SVN rm dir
$SVN commit -m 'Removed dir on trunk'

# Add the dir and file on branch.
$SVN switch ^/branches/branch
mkdir dir
echo heybranch >dir/file.txt
$SVN add dir
$SVN commit -m 'Created dir/file.txt on branch'
$SVN update

# Merge from trunk.
$SVN merge --non-interactive ^/trunk || true

# The file conflicts.  Resolve in favor of working dir.
$SVN resolve --accept=working dir

$SVN propget svn:mergeinfo -R . >actual.out
echo '. - /trunk:2-5' >expected.out

$SVN --version | head -1

if diff -q actual.out expected.out; then
    echo Success
else
    echo 'Test failed!'
    echo 'Expected output:'
    cat expected.out
    echo 'Actual output:'
    cat actual.out
    exit 1
fi
